# Metrics make the world go round.
# Yes, this is a reference to Hitchhikers Guide to the Galaxy.

"""
scripted access to the metrics database views for data analysts
"""

import argparse
import datetime
import pprint
from typing import Dict

import psycopg2
import psycopg2.extras
import logging
import sys
from pycapo import CapoConfig

logger = logging.getLogger("ws_metrics")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class LifeUniverseEverything:

    def __init__(self, connection, args: argparse.Namespace):
        self.conn = connection
        self.args = args

    def get_total_cap_executions(self, capability_name: str):
        sql = "SELECT count(*) From capability_execution_metrics WHERE cap_name = \'" + capability_name[0] + "\'"
        cursor = self.conn.cursor()
        cursor.execute(sql)
        total_number = cursor.fetchone()
        print("SQL query used: " + sql)
        return total_number

    def get_total_executions_in_range(self,
                                      capability_name: str,
                                      beginning_date: datetime,
                                      end_date: datetime):
        sql = "SELECT count(*) FROM capability_execution_metrics WHERE cap_name = \'" + capability_name +\
              "\' AND execution_created between %(beginning)s and %(end)s"
        cursor = self.conn.cursor()
        cursor.execute(sql, {'beginning': beginning_date, 'end': end_date})
        total_number = cursor.fetchone()
        print("SQL query used: " + sql)
        return total_number

    def get_request_data_volume(self, request_id: str):
        sql = "SELECT sum(datasize) FROM download_metrics WHERE request_id = " + request_id +\
              " AND state = \'Complete\' GROUP BY request_id"
        cursor = self.conn.cursor()
        cursor.execute(sql)
        data_volume = cursor.fetchone()
        print("SQL query used: " + sql)
        return data_volume

    def get_total_data_volume(self,
                              beginning_date: datetime,
                              end_date: datetime):
        sql = "SELECT sum(datasize) FROM download_metrics WHERE state = \'Complete\' " \
              "AND execution_created between %(beginning)s and %(end)s"
        cursor = self.conn.cursor()
        cursor.execute(sql, {'beginning': beginning_date, 'end': end_date})
        data_volume = cursor.fetchone()
        print("SQL query used: " + sql)
        return data_volume

    def run_metrics(self, args: argparse.Namespace):
        if args.capability:
            result = self.get_total_cap_executions(args.capability)
            print("\nTotal number of " + args.capability[0] + " capability executions: " + str(result[0]) + "\n")
        if args.between:
            result = self.get_total_executions_in_range(args.between[0], args.between[1], args.between[2])
            print("\nTotal number of " + args.between[0] +
                  " executions between " + args.between[1] +
                  " and " + args.between[2] + ": " + str(result[0]) + "\n")
        if args.requestdatasize:
            result = self.get_request_data_volume(args.requestdatasize[0])
            print(result)
            if result is not None:
                print("\nDownload data size for request #" + args.requestdatasize[0] + " : " + str(result[0]))
            else:
                print("Request #" + args.requestdatasize[0] + " not found!")
        if args.datasize:
            result = self.get_total_data_volume(args.datasize[0], args.datasize[1])
            print(result)
            if result[0] is not None:
                print("\nTotal volume of downloaded data for range " + args.datasize[0] + " to " +
                      args.datasize[1] + ": " + str(result[0]))
            else:
                print("Error: No download capability requests were found!")


def parser() -> argparse.ArgumentParser:
    """
    Initialize argument parser for command-line arguments

    :return: Argument parser
    """
    parser = argparse.ArgumentParser(
        description="Workspaces metrics reporter",
        formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        "-c",
        "--capability",
        nargs=1,
        action="store",
        required=False,
        help="report number of times a capability has executed"
    )
    parser.add_argument(
        "-b",
        "--between",
        nargs=3,
        action="store",
        required=False,
        help="report the number of times a capability has executed between two user provided timestamps."
    )
    parser.add_argument(
        "-rd",
        "--requestdatasize",
        nargs=1,
        action="store",
        required=False,
        help="reports the volume of data downloaded for this request in kilobytes."
    )
    parser.add_argument(
        "-d",
        "--datasize",
        nargs=2,
        action="store",
        required=False,
        help="reports the total volume of data downloaded via the workspaces system over a specified period of time."
    )

    return parser


def _get_capo_settings():
    return CapoConfig().settings("metadataDatabase")


def main():
    print("**********************************")
    print("* WELCOME TO WORKSPACES METRICS! *")
    print("**********************************")
    args = parser().parse_args()
    settings = _get_capo_settings()
    host, dbname = settings['jdbcUrl'].split(':')[2][2:].split('/')
    try:
        conn = psycopg2.connect(
            host=host,
            database=dbname,
            user=settings.jdbcUsername,
            password=settings.jdbcPassword
        )
    except:
        print("Unable to connect to database.")

    LifeUniverseEverything(conn, args).run_metrics(args)
    conn.close()

